@@ -118,8 +118,8 @@ class GenericRPCTests(object):
 
         self.rpc(filename, source, offset)
 
-    @unittest.skipIf((3, 3) <= sys.version_info < (3, 4),
-                     "Bug in jedi for Python 3.3")
+    # @unittest.skipIf((3, 3) <= sys.version_info < (3, 4),
+    #                  "Bug in jedi for Python 3.3")
     def test_should_not_fail_for_relative_import(self):
         source, offset = source_and_offset(
             "from .. import foo_|_"
@@ -343,7 +343,7 @@ class RPCGetCompletionsTests(GenericRPCTests):
         for candidate in expected:
             self.assertIn(candidate, actual)
 
-    if sys.version_info >= (3, 5):
+    if sys.version_info >= (3, 5) or sys.version_info < (3, 0):
         JSON_COMPLETIONS = ["SONDecoder", "SONEncoder", "SONDecodeError"]
     else:
         JSON_COMPLETIONS = ["SONDecoder", "SONEncoder"]
@@ -378,8 +378,12 @@ class RPCGetCompletionsTests(GenericRPCTests):
         completions = self.backend.rpc_get_completions(filename,
                                                        source,
                                                        offset)
+        if sys.version_info < (3, 0):
+            compl = [u'me', u'METext']
+        else:
+            compl = ['me']
         self.assertEqual([cand['suffix'] for cand in completions],
-                         ["me"])
+                         compl)
 
     def test_should_not_complete_for_import(self):
         source, offset = source_and_offset("import foo.Conf_|_")
@@ -390,8 +394,8 @@ class RPCGetCompletionsTests(GenericRPCTests):
         self.assertEqual([cand['suffix'] for cand in completions],
                          [])
 
-    @unittest.skipIf((3, 3) <= sys.version_info < (3, 4),
-                     "Bug in jedi for Python 3.3")
+    # @unittest.skipIf((3, 3) <= sys.version_info < (3, 4),
+    #                  "Bug in jedi for Python 3.3")
     def test_should_not_fail_for_short_module(self):
         source, offset = source_and_offset("from .. import foo_|_")
         filename = self.project_file("test.py", source)
@@ -692,8 +696,6 @@ class RPCGetAssignmentTests(GenericRPCTests):
 class RPCGetCalltipTests(GenericRPCTests):
     METHOD = "rpc_get_calltip"
 
-    @unittest.skipIf(sys.version_info >= (3, 0),
-                     "Bug in Jedi 0.9.0")
     def test_should_get_calltip(self):
         source, offset = source_and_offset(
             "import threading\nthreading.Thread(_|_")
@@ -706,8 +708,6 @@ class RPCGetCalltipTests(GenericRPCTests):
 
         self.assertEqual(calltip, expected)
 
-    @unittest.skipIf(sys.version_info >= (3, 0),
-                     "Bug in Jedi 0.9.0")
     def test_should_get_calltip_even_after_parens(self):
         source, offset = source_and_offset(
             "import threading\nthreading.Thread(foo()_|_")
@@ -719,8 +719,6 @@ class RPCGetCalltipTests(GenericRPCTests):
 
         self.assertEqual(self.THREAD_CALLTIP, actual)
 
-    @unittest.skipIf(sys.version_info >= (3, 0),
-                     "Bug in Jedi 0.9.0")
     def test_should_get_calltip_at_closing_paren(self):
         source, offset = source_and_offset(
             "import threading\nthreading.Thread(_|_)")
@@ -833,6 +831,47 @@ class RPCGetDocstringTests(GenericRPCTests):
         self.assertIsNone(docstring)
 
 
+class RPCGetOnelineDocstringTests(GenericRPCTests):
+    METHOD = "rpc_get_oneline_docstring"
+
+    def check_docstring(self, docstring):
+
+        self.assertEqual(docstring['doc'],
+                         self.JSON_LOADS_DOCSTRING)
+
+    def check_module_docstring(self, docstring):
+
+        self.assertEqual(docstring['doc'],
+                         self.JSON_DOCSTRING)
+
+    def test_should_get_oneline_docstring(self):
+        source, offset = source_and_offset(
+            "import json\njson.loads_|_(")
+        filename = self.project_file("test.py", source)
+        docstring = self.backend.rpc_get_oneline_docstring(filename,
+                                                           source,
+                                                           offset)
+        self.check_docstring(docstring)
+
+    def test_should_get_oneline_docstring_for_modules(self):
+        source, offset = source_and_offset(
+            "import json_|_\njson.loads(")
+        filename = self.project_file("test.py", source)
+        docstring = self.backend.rpc_get_oneline_docstring(filename,
+                                                           source,
+                                                           offset)
+        self.check_module_docstring(docstring)
+
+    def test_should_return_none_for_bad_identifier(self):
+        source, offset = source_and_offset(
+            "froblgoo_|_(\n")
+        filename = self.project_file("test.py", source)
+        docstring = self.backend.rpc_get_oneline_docstring(filename,
+                                                           source,
+                                                           offset)
+        self.assertIsNone(docstring)
+
+
 class RPCGetNamesTests(GenericRPCTests):
     METHOD = "rpc_get_names"
 
