@@ -16,26 +16,23 @@
 
 (ert-deftest elpy-company-backend-should-find-no-prefix-in-string ()
   (elpy-testcase ()
-    (elpy-modules-run 'global-init)
+    (elpy-enable)
     (python-mode)
-    (elpy-mode)
     (insert "# hello")
     (should-not (elpy-company-backend 'prefix))))
 
 (ert-deftest elpy-company-backend-should-find-simple-prefix-string ()
   (elpy-testcase ()
-    (elpy-modules-run 'global-init)
+    (elpy-enable)
     (python-mode)
-    (elpy-mode)
     (insert "hello")
     (should (equal (elpy-company-backend 'prefix)
                    "hello"))))
 
 (ert-deftest elpy-company-backend-should-find-full-prefix-string ()
   (elpy-testcase ()
-    (elpy-modules-run 'global-init)
+    (elpy-enable)
     (python-mode)
-    (elpy-mode)
     (insert "hello.world")
     (should (equal (elpy-company-backend 'prefix)
                    '("world" . t)))))
@@ -120,9 +117,8 @@
 
 (ert-deftest elpy-company-backend-should-add-shell-candidates ()
   (elpy-testcase ()
-    (elpy-modules-run 'global-init)
+    (elpy-enable)
     (python-mode)
-    (elpy-mode)
     (let ((elpy-get-info-from-shell t)
           (elpy-get-info-from-shell-timeout 10))
       (insert "variable_script = 3\n")
@@ -142,7 +138,7 @@
                          "variable_shell variable_script")))
       ;; Test function completions
       (insert "\nfunction")
-      (sleep-for 1)
+      (sleep-for 0.1)
       (let* ((cand (elpy-rpc-get-completions))
              (ext-cand (elpy-company--add-interpreter-completions-candidates cand)))
         (should (string= (mapconcat (lambda (cand) (cdr (assoc 'name cand)))
@@ -154,9 +150,8 @@
 
 (ert-deftest elpy-company-backend-should-not-add-shell-candidates ()
   (elpy-testcase ()
-    (elpy-modules-run 'global-init)
+    (elpy-enable)
     (python-mode)
-    (elpy-mode)
     (insert "variable_script = 3\n")
     (insert "def function_script(a): print(a)\n")
     (elpy-shell-get-or-create-process)
