@@ -14,11 +14,11 @@
 (ert-deftest elpy-config-should-show-flake8-pip-button-when-no-syntax-checker-available ()
   (elpy-testcase ()
                  (mletf* ((elpy-syntax-check-command "/foo/bar/flake8"))
+                         (elpy-enable)
                          (python-mode)
-                         (elpy-mode)
                          (elpy-config)
                          (let ((output (with-current-buffer  "*Elpy Config*"
-                                         (elpy/wait-for-output "Only")
+                                         (elpy/wait-for-output "Options")
                                          (buffer-string))))
                            (should (equal elpy-syntax-check-command "/foo/bar/flake8"))
-                           (should (string-match "python.*\ -m pip install.*flake8" output))))))
+                           (should (string-match "Install flake8" output))))))
