@@ -1,8 +1,8 @@
 (ert-deftest elpy-xref--get-completion-table-should-return-completion-table ()
   (when (featurep 'xref)
     (elpy-testcase ((:project project-root "test.py")) (find-file (f-join project-root "test.py"))
+        (elpy-enable)
         (python-mode)
-        (elpy-mode)
         (insert "def foo(x, y):\n"
                 "    return x + y\n"
                 "var1 = foo(5, 2)")
